%________________THE MODEL_________________________________________
clc
clear all
scrsz = get(0,'ScreenSize');

P=1:2:70; %Polar axis
E=P; %Equatorial axis
m_P=meshgrid(P,E); m_E=m_P';
stick_r=0.05; %Sticking surface area is the function of stick_r (see. description in manuscript for details)

P2E=m_P./m_E;

disp('Calculation of sitcking surface area...')
%Numerical calculation of sitcking surface area
S_num_zbio=num_int_ell(m_P,m_E,stick_r);


disp(' ')

c=m_P./2; a=m_E./2; %radii
cmx=max(P)./2; amx=max(E)./2; cmi=min(P)./2; ami=min(E)./2;
S=zeros(size(a)); e1=NaN(size(a));
V=4/3.*pi.*c.*a.*a;
%Prolate shape c>a
e1(a<c)=sqrt(1-(a(a<c).^2./c(a<c).^2)); %a<c prolate shape
S(a<c)=2.*pi.*a(a<c).^2+(2.*pi.*a(a<c).*c(a<c)./e1(a<c)).*(asin(e1(a<c)));
%Oblate shape c<a
e1(a>c)=sqrt(1-(c(a>c).^2./a(a>c).^2)); %a>c oblate shape
S(a>c)=(2.*pi.*a(a>c).^2+pi.*c(a>c).^2./e1(a>c).*log((1+e1(a>c))./(1-e1(a>c))));
%Sphere c==a
S(a==c)=4.*pi.*a(a==c).^2;

if (cmx==amx)+(cmi==ami)==2 %#ok sphere
    Smx=4*pi*cmx^2; Smi=4*pi*cmi^2;
else
    error('set max(P) == max(E)')
end
Vmx=4/3*pi*cmx*amx*amx; Vmi=4/3*pi*cmi*ami*ami;


%________Variables___________________
%V - pollen volume, V0 - pollen volume w chwili t0
%S - Surface area
%T - temperature
%teta0 = relative water content at t0
%teta_crit = critical relative water content
%alpha - power in formula for C
%a1 = water content at start of germination process
teta0=0.5;
teta_crit=0.06;
T_v_p=[15 30]; %Temperature
T_v_p=sort(T_v_p);
alpha_v_p=[1 0.95 0.9];
Sn=0.01;
Rh=30; % relative humidity [%]
k=10^-5; %conductace for water loss coeficient
a1=0.3;
a6=10^-7;

%pj log distributed; p match the chance that grain is delivered to stigma
%during some small time interval
p=10.^(-14:0.5:-13);

q=(Sn/(1-Sn))./(p+(Sn/(1-Sn))); %the chance that grain will not fall from the insect
%to the stigma and be lost during one time interval

s_alpv=length(alpha_v_p);
s_Tv=length(T_v_p);
s_qjv=length(q);

pj_v=repmat(p,[1,s_Tv*s_alpv]);
qj_v=repmat(q,[1,s_Tv*s_alpv]);
alpha_v=sort(repmat(alpha_v_p,[1,s_Tv*s_qjv]));
T_v=repmat(sort(repmat(T_v_p,[1,s_qjv])),[1,s_alpv]);

%Resutls:
wyn_poll=NaN(28,length(alpha_v));

disp('Calculation of expected time of pollen viability...')

L_matr=repmat(zeros(size(V)),[1,1,length(T_v_p)]); %expected time of grain viability
ne=numel(V)*length(T_v_p);

options = odeset('RelTol', 1e-12); %ode113 solver accuraccy
coo=0;
for gg3=1:length(T_v_p)
    T=T_v_p(gg3);
    p0=6.112*10^2*exp((17.67.*T)./(T+243.5)); %water vapour presure in Pa
    for gg2=1:length(P)
        for gg1=1:length(E)
            coo=coo+1;
            when_death=[];
            L1_=1;  L2_=10^10; co_=5;
            while isempty(when_death)
                L2_=L2_*5;
                co_=co_+1;
                [t,w]=ode113(@(t,teta) dehydration(t,teta,S(gg1,gg2),V(gg1,gg2),T,k,p0,Rh),[L1_ L2_],teta0,options);
                when_death=find(w<teta_crit,1,'first');
                if co_>25
                    disp('It seems that water content will not fall below teta_crit')
                    figure; plot(t,w)
                end
            end
            L1_=t(when_death-1); L2_=t(when_death);
            w_at_L1=w(when_death-1); when_death=[];
            for jj=1:3
                while isempty(when_death)
                    [t,w]=ode113(@(t,teta) dehydration(t,teta,S(gg1,gg2),V(gg1,gg2),T,k,p0,Rh),[L1_ L1_+1:(L2_-L1_)/100:L2_-1 L2_ ],w_at_L1,options);
                    when_death=find(w<teta_crit,1,'first');
                    L2_=L2_+(L2_-L1_);
                end
                L1_=t(when_death-1); L2_=t(when_death);
            end
            L_matr(gg1,gg2,gg3)=t(when_death);
        end
        disp(ne-coo)
    end
end


%Obliczanie pj dla wektora qj i zalozonego Sn
for i_poll=1:length(alpha_v)
    disp(num2str(i_poll./length(alpha_v)))
    alpha_j=alpha_v(i_poll);
    T=T_v(i_poll);
    qj=qj_v(i_poll);
    pj=pj_v(i_poll);
   
    %##############  L  ##############
    L=L_matr(:,:,find(T_v_p==T)); %#ok    
    %##############  Pollination chance  ##############
    P_zap=(pj*qj).*((((qj*(1-pj)).^(fix(L)+1))-1)./(qj*(1-pj)-1));
    %##############  N  ##############  - grains number
    En=10^10; %Energy pool available for reproduction
    N=En./V;
    %##############  G  ############## - germination potential
    G=(V-a1.*V)./(S_num_zbio);

    %##############  C  ##############- competition potential
    C=1./(a6.*V.^alpha_j);

    tc=C+G;
    Germ_and_Grow=1./tc;

    %Fitness
    F=N.*Germ_and_Grow.*P_zap;

    [r1,c1]=find(F==max(max(F(P2E<1))));
    [r2,c2]=find(F==max(max(F(P2E>1))));

    %Fitness and fitness components
    figure
    plot3(m_P(m_P==m_E),m_E(m_P==m_E),log10(N(m_P==m_E)),'LineWidth',3,'Color','r')
    hold on
    surf(P,E,log10(N)); xlabel('P'); ylabel('E'); zlabel('log_1_0(N)');
    grid on;

    figure
    plot3(m_P(m_P==m_E),m_E(m_P==m_E),P_zap(m_P==m_E),'LineWidth',3,'Color','r')
    hold on
    surf(m_P,m_E,P_zap); xlabel('P'); ylabel('E'); zlabel('P zap');
    grid on;

    figure
    plot3(m_P(m_P==m_E),m_E(m_P==m_E),F(m_P==m_E),'LineWidth',3,'Color','r')
    hold on
    surf(m_P  ,m_E,F); xlabel('P'); ylabel('E'); zlabel('F');
    grid on;
    title(['T = ',num2str(T),'  log_1_0(pj) = ',num2str(log10(pj)),'  alpha = ',num2str(alpha_j)])

    pause
    close all

    naglowki={'alpha';'pj';'T';'a6';'P oblate';'P prolate';'E oblate';'E prolate';...
        'P2E oblate';'P2E prolate';'V oblate';'V prolate';...
        'S oblate';'S prolate';'x1S oblate';'x1S prolate';...
        'N oblate';'N prolate';'Germ time [G] oblate';'Germ time [G] prolate';...
        'Grow time [C] oblate';'Grow time [C] prolate';...
        'Germ_and_Grow oblate';'Germ_and_Grow prolate';...
        'P_zap oblate';'P_zap prolate';'Fit oblate';'Fit prolate'};

    wyn_poll(1,i_poll)=alpha_j;
    wyn_poll(2,i_poll)=pj;
    wyn_poll(3,i_poll)=T;
    wyn_poll(4,i_poll)=a6;

    wyn_poll(5,i_poll)=m_P(r1,c1); %P oblate
    wyn_poll(6,i_poll)=m_P(r2,c2); %P prolate
    wyn_poll(7,i_poll)=m_E(r1,c1); %E oblate
    wyn_poll(8,i_poll)=m_E(r2,c2); %E prolate
    wyn_poll(9,i_poll)=m_P(r1,c1)./m_E(r1,c1); %P2E oblate
    wyn_poll(10,i_poll)=m_P(r2,c2)./m_E(r2,c2); %P2E prolate
    wyn_poll(11,i_poll)=V(r1,c1); %V oblate
    wyn_poll(12,i_poll)=V(r2,c2); %V prolate
    wyn_poll(13,i_poll)=S(r1,c1); %S oblate
    wyn_poll(14,i_poll)=S(r2,c2); %S prolate
    wyn_poll(15,i_poll)=S_num_zbio(r1,c1); %x1S oblate
    wyn_poll(16,i_poll)=S_num_zbio(r2,c2); %x1S prolate
    wyn_poll(17,i_poll)=N(r1,c1); %N oblate
    wyn_poll(18,i_poll)=N(r2,c2); %N prolate
    wyn_poll(19,i_poll)=G(r1,c1); %Germ time oblate
    wyn_poll(20,i_poll)=G(r2,c2); %Germ time prolate
    wyn_poll(21,i_poll)=C(r1,c1); %Grow time oblate
    wyn_poll(22,i_poll)=C(r2,c2); %Grow time prolate
    wyn_poll(23,i_poll)=Germ_and_Grow(r1,c1); %Germ_and_Grow oblate
    wyn_poll(24,i_poll)=Germ_and_Grow(r2,c2); %Germ_and_Grow prolate
    wyn_poll(25,i_poll)=P_zap(r1,c1); %P_zap oblate
    wyn_poll(26,i_poll)=P_zap(r2,c2); %P_zap prolate
    wyn_poll(27,i_poll)=F(r1,c1); %Fit oblate
    wyn_poll(28,i_poll)=F(r2,c2); %Fit prolate
end
